#!/bin/bash

# Alias for listing scheduled jobs
alias salt-schedule-list='docker exec -it salt_master_1 salt-call schedule.list'

# Alias for adding a scheduled job
alias salt-schedule-add='docker exec -it salt_master_1 salt-call schedule.add'

# Alias for viewing job history
alias salt-job-history='docker exec -it salt_master_1 salt-run jobs.list_jobs'

# Alias for looking up a specific job
alias salt-job-lookup='docker exec -it salt_master_1 salt-run jobs.lookup_jid'

# Alias for real-time event monitoring
alias salt-event-monitor='docker exec -it salt_master_1 salt-run state.event pretty=True'

# Function to schedule an orchestration job
function salt-schedule-orchestration {
  if [ "$#" -ne 5 ]; then
    echo "Usage: salt-schedule-orchestration <job_name> <orchestration_sls> <start_time> <end_time> <interval_minutes>"
    echo "Example: salt-schedule-orchestration apply_specific_states orchestrate.apply_specific_states '0 9 * * 6' '0 17 * * 6' 60"
    return 1
  fi

  local job_name=$1
  local orchestration_sls=$2
  local start_time=$3
  local end_time=$4
  local interval_minutes=$5

  docker exec -it salt_master_1 salt-call schedule.add $job_name \
    function='state.orchestrate' \
    job_args="[\"$orchestration_sls\"]" \
    when="$start_time" \
    until="$end_time" \
    minutes=$interval_minutes
}

# Function to monitor the last job executed on minions
function salt-minion-last-job {
  if [ "$#" -ne 1 ]; then
    echo "Usage: salt-minion-last-job <minion_id>"
    echo "Example: salt-minion-last-job minion_id"
    return 1
  fi

  local minion_id=$1
  docker exec -it salt_master_1 salt $minion_id saltutil.last_job
}

# Function to find a specific job on a minion
function salt-minion-find-job {
  if [ "$#" -ne 2 ]; then
    echo "Usage: salt-minion-find-job <minion_id> <job_id>"
    echo "Example: salt-minion-find-job minion_id job_id"
    return 1
  fi

  local minion_id=$1
  local job_id=$2
  docker exec -it salt_master_1 salt $minion_id saltutil.find_job $job_id
}

# Function to print usage and options
function salt-macros {
  echo "Available Salt macros and their usage:"
  echo
  echo "Aliases:"
  echo "  salt-schedule-list          - List scheduled jobs"
  echo "  salt-schedule-add           - Add a scheduled job"
  echo "  salt-job-history            - View job history"
  echo "  salt-job-lookup <job_id>    - Look up a specific job"
  echo "  salt-event-monitor          - Monitor Salt events in real-time"
  echo
  echo "Functions:"
  echo "  salt-schedule-orchestration <job_name> <orchestration_sls> <start_time> <end_time> <interval_minutes>"
  echo "    - Schedule an orchestration job"
  echo "    - Example: salt-schedule-orchestration apply_specific_states orchestrate.apply_specific_states '0 9 * * 6' '0 17 * * 6' 60"
  echo
  echo "  salt-minion-last-job <minion_id>"
  echo "    - Monitor the last job executed on a minion"
  echo "    - Example: salt-minion-last-job minion_id"
  echo
  echo "  salt-minion-find-job <minion_id> <job_id>"
  echo "    - Find a specific job on a minion"
  echo "    - Example: salt-minion-find-job minion_id job_id"
}

# Example usage suggestions
# To schedule an orchestration:
# salt-schedule-orchestration "apply_specific_states" "orchestrate.apply_specific_states" "0 9 * * 6" "0 17 * * 6" 60

# To monitor the last job on a minion:
# salt-minion-last-job "minion_id"

# To find a specific job on a minion:
# salt-minion-find-job "minion_id" "job_id"

